%% example implementation
% 'Using High-Level Visual Information for Color Constancy'
% J. van de Weijer, C. Schmid, J. Verbeek
% ICCV 2007, Rio-de-Janeiro

% settings of BOW representation images
NC=1000;                % number of color words
NS=750;                 % number of shape words
Npos=8;                 % number of horizontal and vertical bins used for position cue
num_samples=50;         % number of samples for vocabulary construction

% settings for the PLSA 
number_of_topics=30;    % number of semantic topics

% the color constancy settings which are considered (important: first setting should always be NN=0, PP=0, SS=0)
%        WP            SoG      GGW     GE1                 GE2
NN=[0   0  0  0  0   0 0 0      0 0     1 1 1 1 1 1 1     2 2 2 2 2 2 2 2];      % order of differentiation
PP=[0  -1 -1 -1 -1   1 2 5      1 5     1 1 1 1 2 3 5     1 1 1 1 1 2 3 5];      % Minkowski norm
SS=[0   0  1  3  5   0 0 0      3 1     1 2 3 4 1 1 1     1 2 3 6 7 2 3 5];      % scale of Gaussian
num_ll=length(NN);      % number of settings considered

% read database settings
init_cc

%% main program
errors=[];
likelihood_I=[];
illuminant_est=[];

for split_counter=1:opts.nSplits            % run the system for all the splits
    testset=(opts.splits==split_counter);
    trainset=1-testset;

    % compute vocabularies and semantic topics 
    compute_vocabularies_cc
    compute_topics_cc
    
    % compute bottom_up, top-down, and combined color constancy estimates
    PLSA_cc 
end

% the mean and median error (the last three numbers (seperated by a zero) are the bottom-up, top-down and the combined BU and TD result; the first numbers are the results for the general_cc methods)
mean(errors)/pi*180
median(errors)/pi*180
