function [wt,td,F_last,pixel_prob] = plsa2(wtP,wtS,wtC,epsilon,plo,learn,mask)
% plsa2 computes the most likely word-topic and topic-document distributions
% given the distribution over position, color and shape.
%
% [wt,td] = plsa(counts,T,epsilon,plo,wt,learn)
%
% learns plsa model with T topics from words x docs counts data
% 
% counts  (W x D)  : matrix with count data of W words for D documents 
% T       (scalar) : desired number of topics 
% epsilon (scalar) : EM iterations terminate if the relative change in log-likelihood is under epsilon
% plo     (scalar) : plot log-likelihood progress and wt and td matrices if plo ~= 0
% wt      (W x T)  : optional, initializes word-topic distributions from wt (columns should normalize to one) 
% learn   (scalar) : keeps wt fixed if learn = 0;
%
% wt      (W x T)  : word-topic distributions
% td      (T x D)  : topic-document distributions
% E	  (1 x D)  : document log-likelihood: log p( words in doc | doc )
%
% J.J. Verbeek, INRIA Rhone-Alpes, 2006 (adapted J.van de Weijer 2007)

if(nargin<7)
    mask_flag=0;
else
    mask_flag=1;
end

T=size(wtS,2);

if(plo), COLORMAP('default'); end;

W=length(wtP);
if(plo) fprintf('--> PLSA started with %d topics, %d documents, and %d word vocabulary.\n',T,1,W);  end

tot=length(wtP);

done = 0;
iter = 1;

wt=wtC.*wtP.*wtS;

td=sum(normalize(wt,2),1)';
if(mask_flag)
    td=normalize(td.*mask,1);
else
    td=normalize(td,1);
end
E = sum( log( wt*td  +eps) ,1);
F = sum( E ) / tot;
    
counts=ones(tot,1);

while ~done; iter = iter+ 1; 
 
   td = normalize( td .* ( wt' * ( counts ./ ( wt * td + eps ) ) ), 1);
   if(mask_flag)
       td=normalize(td.*mask,1);
   else
       td=normalize(td,1);
   end
   
   if learn;
       display(' SHOULD BE ADJUSTED !!');   %see plsa_test2
   end
         
   E = sum( log( wt*td  +eps) ,1);
   F_new = sum( E ) / tot;F =  [F F_new];
       
   rel_ch = (F(iter) - F(iter-1))/abs(F(iter-1)); if rel_ch< epsilon && (iter> 5 ); done=1;end
   
   if plo;
      fprintf('iteration %3d F = %f rel-ch = %.6f \n', iter, F(iter), rel_ch); 
  
      Nplots = 3;    
      set(gcf,'Name','Probabilistic Latent Semantic Analysis');
      subplot(Nplots,1,1); imagesc(td);         title('Distribution over topics per document');
      subplot(Nplots,1,2); imagesc(wt');        title('Distribution over words per topic');
      subplot(Nplots,1,3); plot(F(2:end),'.-'); title('Average data log-likelihood per iteration')
      drawnow;
   end   
end

F_last=sum( log( wt*td  +eps) ,1)/tot;
pixel_prob=log( wt*td  +eps);